"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SplTokenMetadata = void 0;
const arweave_1 = require("./arweave");
const metadata_1 = require("./metadata");
const web3_js_1 = require("@solana/web3.js");
const _1 = require(".");
class SplTokenMetadata {
    constructor(opts) {
        this.provider = opts.provider;
    }
    static init(provider) {
        return __awaiter(this, void 0, void 0, function* () {
            return new this({
                provider
            });
        });
    }
    sendInstructions(instructions, signers, payer) {
        return (0, _1.sendInstructions)(new Map(), this.provider, instructions, signers, payer);
    }
    presignCreateArweaveUrlInstructions({ name, symbol, description = "", image, creators, files = [], payer = this.provider.wallet.publicKey }) {
        return __awaiter(this, void 0, void 0, function* () {
            const metadata = {
                name,
                symbol,
                description,
                image,
                external_url: "",
                animation_url: undefined,
                properties: {
                    category: metadata_1.MetadataCategory.Image,
                    files,
                },
                creators: creators ? creators : null,
                sellerFeeBasisPoints: 0,
            };
            const realFiles = (0, arweave_1.getFilesWithMetadata)(files, metadata);
            const prepayTxnInstructions = yield (0, arweave_1.prepPayForFilesInstructions)(payer, realFiles);
            return {
                instructions: prepayTxnInstructions,
                signers: [],
                output: {
                    files: realFiles
                }
            };
        });
    }
    presignCreateArweaveUrl(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { output: { files }, instructions, signers, } = yield this.presignCreateArweaveUrlInstructions(args);
            const txid = yield this.sendInstructions(instructions, signers);
            return {
                files,
                txid
            };
        });
    }
    getArweaveUrl({ txid, mint, files = [], uploadUrl = arweave_1.ARWEAVE_UPLOAD_URL }) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, arweave_1.uploadToArweave)(txid, mint, files, uploadUrl);
            const metadataFile = (_a = result.messages) === null || _a === void 0 ? void 0 : _a.find((m) => m.filename === "manifest.json");
            console.log(JSON.stringify(metadataFile, null, 2));
            if (!metadataFile) {
                throw new Error("Metdata file not found");
            }
            // Use the uploaded arweave files in token metadata
            return `https://arweave.net/${metadataFile.transactionId}`;
        });
    }
    createMetadataInstructions({ data, authority = this.provider.wallet.publicKey, mint, payer = this.provider.wallet.publicKey }) {
        return __awaiter(this, void 0, void 0, function* () {
            const instructions = [];
            const metadata = yield (0, metadata_1.createMetadata)(data, authority === null || authority === void 0 ? void 0 : authority.toBase58(), mint.toBase58(), payer.toBase58(), instructions, payer.toBase58());
            return {
                instructions,
                signers: [],
                output: {
                    metadata: new web3_js_1.PublicKey(metadata)
                }
            };
        });
    }
}
exports.SplTokenMetadata = SplTokenMetadata;
//# sourceMappingURL=splTokenMetadata.js.map